#! /bin/bash

# This script builds the CoCoA-5 GUI; it assumes that CoCoAInterpreter
# has already been built, and that the Qt libraries have been installed
# (and that the qmake command is available).

if [ $# -ne 4 ]
then
  echo "ERROR: $0 requires 4 args (full paths of BOOST_DIR, libgmpxx, libgmp, gmp.h)"  > /dev/stderr
  exit 1
fi

BOOST_DIR=$1
GMPXX_LIB=$2
GMP_LIB=$3
GMP_INC_DIR=$4

if [ \! -d "$BOOST_DIR" -o \! -r "$BOOST_DIR" ]
then
  echo "ERROR: $0 failed because cannot access BOOST in directory $BOOST_DIR"  >/dev/stderr
  exit 1
fi
BOOST_LIB=`dirname "$BOOST_DIR"`/lib
if [ \! -f "$BOOST_LIB/libboost_thread.a" ]
then
  echo "ERROR: $0 failed because cannot find libboost_thread.a in directory $BOOST_LIB"  >/dev/stderr
  exit 1
fi

if [ \! -f "$GMPXX_LIB" ]
then
  echo "ERROR: $0 failed because $GMPXX_LIB is not a file" >/dev/stderr
fi
if [ \! -f "$GMP_LIB" ]
then
  echo "ERROR: $0 failed because $GMPXX_LIB is not a file" >/dev/stderr
fi
if [ \! -d "$GMP_INC_DIR" -o \! -r "$GMP_INC_DIR" ]
then
  echo "ERROR: $0 failed because cannot access GMP in directory $GMP_INC_DIR"  >/dev/stderr
  exit 1
fi

SYS_TYPE=`uname`

# On MacOSX we must explicitly state the "architecture".
# The block below should pick the right one (as dictated by libgmp).
if [ "$SYS_TYPE" = "Darwin" ]
then
  ARCH=`arch`  # either "i386" or "ppc"
  if [ "$ARCH" = "i386" ]
  then
    (cd ../../examples; make ex-empty) > /dev/null 2>&1
    arch -x86_64 ../../examples/ex-empty > /dev/null 2>&1
    if [ $? = 0 ]
    then
      ARCH=x86_64
    fi
  fi
  if [ "$ARCH" = "ppc" ]
  then
    (cd ../../examples; make ex-empty) > /dev/null 2>&1
    arch -ppc64 ../../examples/ex-empty > /dev/null 2>&1
    if [ $? = 0 ]
    then
      ARCH=ppc64
    fi
  fi
fi

which qmake >/dev/null
if [ $? -ne 0 ]
then
  echo "ERROR: $0 failed because it cannot find qmake" >/dev/stderr
  exit 1
fi

##################################################################
# Sort out QCodeEdit (which is in subdir QCodeEdit/)

QCE="QCodeEdit"
if [ \! -d "$QCE" ]
then
  echo "ERROR: $0 cannot find subdirectory $QCE"
  exit 2
fi

# if QCodeEdit.pro exists and is newer than QCodeEdit.pro.in
# we could skip rebuilding it
# if [ \! $QCE/QCodeEdit.pro -nt $QCE/QCodeEdit.pro.in ]
# then
/bin/rm -f $QCE/QCodeEdit.pro

if [ "$SYS_TYPE" = "Darwin" ]
then
  echo "CONFIG += $ARCH    #MacOSX architecture" > $QCE/QCodeEdit.pro
fi

cat $QCE/QCodeEdit.pro.in >> $QCE/QCodeEdit.pro
# fi

if [ "$SYS_TYPE" = "Darwin" ]
then
  (cd $QCE; qmake -spec macx-g++ QCodeEdit.pro)
  QMAKE_OK=$?
else
  (cd $QCE; qmake QCodeEdit.pro)
  QMAKE_OK=$?
fi
if [ "$QMAKE_OK" -ne 0 ]
then
  echo "ERROR: $0 failed because qmake failed for QCodeEdit"  >/dev/stderr
  exit 4
fi
(cd $QCE; make -j2)
if [ $? -ne 0 ]
then
  echo "ERROR: $0 failed because make failed for QCodeEdit"  >/dev/stderr
  exit 5
fi

##################################################################
# Build of QCodeEdit was successful, so now try to build C5 GUI

# if C5.pro exists and is newer than C5.pro.in
# we could skip rebuilding it
# if [ \! C5.pro -nt C5.pro.in ]
# then
/bin/rm -f C5.pro

echo "INCLUDEPATH += $BOOST_DIR/boost ../../include"   > C5.pro
echo "INCLUDEPATH += $GMP_INC_DIR"                    >> C5.pro
echo "LIBS += ../../lib/libcocoa.a"                   >> C5.pro
echo "LIBS += \"$GMPXX_LIB\""                         >> C5.pro
echo "LIBS += \"$GMP_LIB\""                           >> C5.pro

if [ "$SYS_TYPE" = "Darwin" ]
then
  echo "CONFIG += $ARCH    #MacOSX architecture"      >> C5.pro
fi
cat C5.pro.in                                         >> C5.pro
# fi

if [ "$SYS_TYPE" = "Darwin" ]
then
  qmake -spec macx-g++ C5.pro
  QMAKE_OK=$?
else
  qmake C5.pro
  QMAKE_OK=$?
fi
if [ "$QMAKE_OK" -ne 0 ]
then
  echo "ERROR: $0 failed because qmake failed for C5.pro"  >/dev/stderr
  exit 6
fi

make -f C5Makefile clean
make -j2 -f C5Makefile 
if [ $? -ne 0 ]
then
  echo "ERROR: $0 failed because make failed for C5"      >/dev/stderr
  exit 7
fi


if [ "$SYS_TYPE" = "Darwin" ]
then
  mv C5.app/Contents/MacOS/C5 C5.app/Contents/MacOS/C5.bin
  cp C5.mac.script C5.app/Contents/MacOS/C5
  PKG_DIR=C5.app/Contents/MacOS/packages
  /bin/rm -rf $PKG_DIR
  mkdir $PKG_DIR
  cp -r packages/*  $PKG_DIR
fi
